 classdef (Abstract) ClusterCriterion < classreg.learning.internal.DisallowVectorOps...
                           & matlab.mixin.CustomDisplay
% ClusterCriterion Clustering evaluation object.
%   ClusterCriterion is an abstract class used clustering evaluation. You
%   cannot create an instance of this class directly. Create an instance of
%   a derived class such as SilhouetteEvaluation, GapEvaluation,
%   CalinskiHarabaszEvaluation or DaviesBouldinEvaluation by calling the
%   function EVALCLUSTERS.

%
%   ClusterCriterion properties:
%      X                     - Data used to create this object.
%      NumObservations       - Number of observations in X.
%      OptimalK              - The optimal number of clusters suggested.
%      OptimalY              - The optimal clustering solution.
%      InspectedK            - List of the number of clusters inspected.
%      CriterionName         - Name of the criterion.
%      CriterionValues       - The criterion values for each number of clusters.
%      ClusteringFunction    - Clustering algorithm used.
%      Missing               - A vector indicating which observations are missing.
%
%   ClusterCriterion methods
%      compact               -  Compact this object.
%      plot                  -  Plot the criterion values.

%   Copyright 2013 The MathWorks, Inc.

properties(GetAccess=public, SetAccess=protected) 
    %X matrix of data used to create this evaluation object.
    %   The X property is a numeric matrix of data with one row per
    %   observation and one column per variable. 
    X;
    
    %InspectedK The list of number of clusters that has been inspected.
    %   The InspectedK property is a vector of integers indicating the
    %   number of clusters to be inspected.
    InspectedK;
    
    %CriterionValues The criterion values.
    %   The CriterionValues property saves the criterion values 
    %   corresponding to each number of clusters in the InspectedK
    %   property.
    CriterionValues;
    
    %CriterionName Name of the criterion. 
    %   The CriterionName property is a string indicating the name of the
    %   criterion requested for evaluation.
    CriterionName;
    
    %OptimalK The suggested number of clusters.
    %   The OptimalK property is a integer indicating the suggested number
    %   of clusters after comparing the values of the requested criterion.
    %   The value of OptimalK is estimated based on the following:
    %   'CalinskiHarabasz'    The number of clusters giving the
    %                         maximum CalinskiHarabasz value.
    %   'silhouette'          The number of clusters giving the maximum
    %                         silhouette value.
    %   'gap'                 The number of clusters giving the first local
    %                         maximum gap value or the global maximum gap
    %                         value within some tolerance.
    %   'DaviesBouldin'       The number of clusters giving the minimum
    %                         DaviesBouldin value.
    OptimalK = nan;
    
    %OptimalY The suggested clustering solution.
    %   The OptimalY property is a column vector saving the clustering
    %   solution corresponding to the value of OptimalK. It will be empty
    %   if the clustering solutions are provided in the input.
    OptimalY;
    
    %ClusteringFunction Clustering algorithm used.
    %   The ClusteringFunction property is empty if the clustering
    %   solutions are provided in the input. Otherwise it's a string or a
    %   function handle representing the clustering algorithm requested.
    ClusteringFunction;
    
    %NumObservations Number of observations.
    %   The NumObservations property gives the number of observations used
    %   when creating the object. This is the number of observations
    %   supplied in the data, minus the number of rows with missing (NaN)
    %   values.
    NumObservations;
    
     
    %Missing A vector indicating which observations are missing.
    %   The Missing property is a column vector of logical values, with
    %   one value for each row in the input data. The value is TRUE if
    %   there are missing (NaN) values in the corresponding row of the
    %   data matrix or clustering solutions.
    Missing;
   
end


properties(GetAccess=protected, SetAccess=protected) 
   FunLoc=[];
    % An integer saving the location of builtin clustering algorithm 
    % in  {'kmeans', 'linkage', 'gmdistribution'} if Y is a string
    % 1 - represents "kmeans". 
    % 2 -represents "linkage"
    % 3 -represents "gmdistribution"
           
   ClustFuncHandle;
   % Either a function handle passed from the input, or a set inside this
   % class based on the string representing the clustering algorithm
   
     PrivX;
    %
  
end

methods (Access = protected,Static);%(Hidden,Static)
    function  [args, dflts] =  getAllArgs()
        args  ={'klist'};
        dflts = {[]};
    end
    
end

 methods (Hidden, Static)
       function a = empty(varargin)
            error(message('stats:clustering:evaluation:ClusterCriterion:NoEmptyAllowed'));
      end
 end
  
  
methods (Access = public)
  function hout = plot(this)
  %PLOT  Plot the requested criterion values 
  %  PLOT(E) plots the requested criterion values versus the number of
  %  clusters.
  %
  %  H = PLOT(E) returns the handle to the line in the plot.
  
        [sortedKList, IDX] = sort(this.InspectedK);
        CritVals = this.CriterionValues;
        critVals = CritVals(IDX);
        h = plot(sortedKList, critVals ,'-ob');
        if ~isnan(this.OptimalK)
            optimalVal = this.CriterionValues(this.InspectedK == this.OptimalK);
            line(this.OptimalK,optimalVal,'Marker','.','MarkerSize',10);
        end
        xlabel(getString(message('stats:clustering:evaluation:ClusterCriterion:PlotXlabel')));
        if nargout>=1
            hout = h;
        end
 end
   
  function this = compact(this)
  %COMPACT Compact this object.
  %   C = COMPACT(S) set the X property, the OptimalY property, the Missing
  %   property to be empty.
  
      this.X = [];
      this.OptimalY=[];
      this.Missing = [];
       
  end
end

methods (Access = protected)
    

function group = getPropertyGroups(obj)
 
  plist1 = struct('NumObservations',   obj.NumObservations,...
                  'InspectedK', obj.InspectedK,...
                  'CriterionValues',obj.CriterionValues,...
                  'OptimalK', obj.OptimalK);
  
  group(1) = matlab.mixin.util.PropertyGroup(plist1);

end

   function [IDX, inspectedK]= soft2hard(this, softY)
        
         p= numel(this.InspectedK);
         IDX = zeros(size(this.X,1),p);
          %convert cell array of matrix (soft solution) to matrix (hard
          %solution)
         inspectedK = this.InspectedK;
         for j=1:p
              [~, IDX(:,j)] = max(softY{j},[],2);
              if numel(unique(IDX(~this.Missing,j)))~= this.InspectedK(j)
                  error(message('stats:clustering:evaluation:ClusterCriterion:Soft2hardBadNC'));
              end
         end
    end
    
    function IDX = evalFun(this,NC)
        try
           IDX = this.ClustFuncHandle(this.PrivX,NC);
          
        catch ME
            if strcmp('MATLAB:UndefinedFunction', ME.identifier) ...
                    && ~isempty(strfind(ME.message, func2str(this.ClustFuncHandle)))
                error(message('stats:clustering:evaluation:ClusterCriterion:FunNotFound', func2str(this.ClustFuncHandle )));
            else
               error(message('stats:clustering:evaluation:ClusterCriterion:FunError', NC, func2str(this.ClustFuncHandle), ME.message));
            end
        end
    end

    
    %check whether the size if valid, convert a soft clustering solution
    %into a hard clustering solution.
    function funcOut = checkFuncHandle(this, funcOut,nc)
        if ~isnumeric(funcOut)
             error(message('stats:clustering:evaluation:ClusterCriterion:FuncOutNonNumeric'));
        end
        
        [nRows, nCols]=size(funcOut);
        if size(this.PrivX,1) ~= nRows
             error(message('stats:clustering:evaluation:ClusterCriterion:FuncOutSizeMismatch'));
        end
        if  any(any(isnan(funcOut)))
            error(message('stats:clustering:evaluation:ClusterCriterion:FuncOutNaN'));
        end
        if nc == 1
            if ~isvector(funcOut)
                error(message('stats:clustering:evaluation:ClusterCriterion:FuncOutBad'));
            end
             if numel(unique(funcOut)) == 1 && all(round(funcOut) == funcOut) && all(funcOut > 0)
                 isHard = true;   %view funcOut as a hard clustering solution
             else
                 isHard = false;  
             end
            
        else % nc ~= 1
            if  nCols == 1  %vector
                if any(round(funcOut) ~= funcOut) || any(funcOut <= 0)
                    error(message('stats:clustering:evaluation:ClusterCriterion:FuncOutBad'));
                end
                
                if numel(unique(funcOut)) ~= nc
                    error(message('stats:clustering:evaluation:ClusterCriterion:FuncOutBad'));
                end
                isHard = true; %hard clustering solution
            else
                if nCols ~= nc
                    error(message('stats:clustering:evaluation:ClusterCriterion:FuncOutBad'));
                end
                isHard = false; %soft score
            end
        end
        
        if ~isHard %convert soft solution to hard solution
           [~, funcOut] = max(funcOut,[],2); 
            if numel(unique(funcOut)) ~= nc
                error(message('stats:clustering:evaluation:ClusterCriterion:Soft2hardBadNC'));
            end
        end
        
    end
    
    %Constructor
    function this = ClusterCriterion(X,Y,klist)
         narginchk(3,inf);
        
         %Basic sanity  checking on X, CLUST, KList
         if ~isnumeric(X) || ~ismatrix(X)
               error(message('stats:clustering:evaluation:ClusterCriterion:XBad'));
         end
         
         this.X=X;
         this.Missing = any(isnan(this.X),2);
       
         if isempty(klist)
             %Y must be precomputed clustering solutions
             if isnumeric(Y) && ismatrix(Y) %hard clustering solution
                 [nYRows, nYCols] = size(Y);
                 if size(X,1) ~= nYRows;
                     error(message('stats:clustering:evaluation:ClusterCriterion:XHardYSizeMismatch'));
                 end
            
                 this.Missing = this.Missing | any(isnan(Y),2);
                 this.InspectedK = zeros(1,nYCols);
            
                 if ~(all(all(Y(~this.Missing,:) > 0)) && all(all(round(Y(~this.Missing,:))==Y(~this.Missing,:))))
                       error(message('stats:clustering:evaluation:ClusterCriterion:BadY'));
                  end
                 for j =1:nYCols
                     this.InspectedK(j) = numel(unique(Y(~this.Missing,j)));
                 end
                                 
             elseif iscell(Y) %soft clustering solution
                 p=numel(Y);
                 this.InspectedK=zeros(1,p);
                 for j=1:p
                     if ~ismatrix(Y{j}) || ~isnumeric(Y{j})
                          error(message('stats:clustering:evaluation:ClusterCriterion:BadY'));
                     end
                     [ny,K]=size(Y{j});
                     if ny ~= size(this.X,1)
                           error(message('stats:clustering:evaluation:ClusterCriterion:XSoftYSizeMismatch'));
                     end
                     this.Missing = this.Missing| any(isnan(Y{j}),2);
                     this.InspectedK(j)=K;      
                 end
                
             else
                  error(message('stats:clustering:evaluation:ClusterCriterion:BadY'));
             end
             
         else  % If KLIST is provided, then Y must be a string or a function handle
            if ~(isnumeric(klist) && isvector(klist) && all(klist > 0) ...
                 && all(round(klist)==klist)) || any(isinf(klist)) || any(isnan(klist))
              error(message('stats:clustering:evaluation:ClusterCriterion:BadKList'));
            end
   
            if ischar(Y);
                ClustFuncList = {'kmeans', 'linkage', 'gmdistribution'};
                
                [Y,this.FunLoc]= internal.stats.getParamVal(Y,ClustFuncList,'CLUST');
                
                if this.FunLoc == 1 %kmeans 
                    this.ClustFuncHandle = ...
                        @(X,NC)(kmeans(X,NC, 'EmptyAction','singleton','rep',5));
                elseif this.FunLoc == 2
                    %Ward linkage with default Euclidean distance will be
                    %used.
                    this.ClustFuncHandle = ...
                        @(X,NC)(clusterdata(X,'linkage','ward','maxclust',NC));
                elseif this.FunLoc == 3
                    this.ClustFuncHandle = ...
                        @(X, NC)(cluster(gmdistribution.fit(X,NC,'rep',5,'sharedcov',true),X));
                end
            elseif isa(Y, 'function_handle')
                this.ClustFuncHandle = Y;
            else
                error(message('stats:clustering:evaluation:ClusterCriterion:BadFun'));
            end
          
            this.ClusteringFunction = Y; %a string or a function handle
            
            this.InspectedK = klist;
         end

         this.NumObservations=sum(~this.Missing,1);
         
         if this.NumObservations == 0
              error(message('stats:clustering:evaluation:ClusterCriterion:NObservationsZero'));
         end
         
         if numel(unique(this.InspectedK)) ~= numel(this.InspectedK)
              error(message('stats:clustering:evaluation:ClusterCriterion:SameK'));
         end
         
         if any(this.Missing)
             this.PrivX = this.X(~this.Missing,:);
         
         else
             this.PrivX = this.X;
         end
         
         this.CriterionValues = nan(1,numel(this.InspectedK));
    end
end


end% classdef

