 classdef   DaviesBouldinEvaluation  < clustering.evaluation.ClusterCriterion
%DaviesBouldinEvaluation Evaluation object based on DaviesBouldin criterion
%   A DaviesBouldinEvaluation object computes the DaviesBouldin criterion
%   values which can be  be used for estimating the number of clusters on
%   data. It's created using the following syntax:
%
%   EVA = EVALCLUSTERS(X,CLUST,'DaviesBouldin')
%
%   DaviesBouldinEvaluation properties:
%      X                     - Data used to create this object.
%      NumObservations       - Number of observations in X.
%      OptimalK              - The optimal number of clusters suggested.
%      OptimalY              - The optimal clustering solution.
%      InspectedK            - List of the number of clusters inspected.
%      CriterionName         - Name of the criterion.
%      CriterionValues       - The DaviesBouldin values for each number of clusters.
%      ClusteringFunction    - Clustering algorithm used.
%      Missing               - A vector indicating which observations are missing.
%
%   DaviesBouldinEvaluation methods:
%      compact               -  Compact this object
%      plot                  -  Plot the criterion values
%
%  See also EVALCLUSTERS, clustering.evaluation.GapEvaluation,
%           clustering.evaluation.SilhouetteEvaluation,
%           clustering.evaluation.CalinskiHarabaszEvaluation,

% References:
%    Davies, D.L., Bouldin, D.W. (1979), A cluster separation measure, IEEE
%    Transactions on Pattern Analysis and Machine Intelligence, Volume 1,
%    Issue 2  p224-227

%   Copyright 2013 The MathWorks, Inc.
  methods
      function hout = plot(this)
         h = plot@clustering.evaluation.ClusterCriterion(this);
         ylabel(getString(message('stats:clustering:evaluation:DaviesBouldinEvaluation:Ylabel')));
         if nargout > 0
             hout = h;
         end
      end
  end
   methods (Hidden)
  function   this = DaviesBouldinEvaluation(X,Y,varargin)
      narginchk(2,inf);
      [basePnames, baseDflts ]= clustering.evaluation.ClusterCriterion.getAllArgs();
       
      klist = ...
      internal.stats.parseArgs(basePnames, baseDflts, varargin{:});
       
      this = this@clustering.evaluation.ClusterCriterion(X,Y,klist);
      this.CriterionName = 'DaviesBouldin';          
           
      %Convert soft score to hard clustering solutions
      if iscell(Y)
          Y = this.soft2hard(Y);
      end
      
      p = numel(this.InspectedK);
      if isnumeric(Y) % Y is hard solution
          Y(this.Missing,:)=[];
          for j=1:p
              this.CriterionValues(j)= getDB(this,Y(:,j));
          end
         
          [minVal,optimalKLoc] = min(this.CriterionValues);
          if ~isnan(minVal)
             this.OptimalK=this.InspectedK(optimalKLoc);
          end
      else%if ischar(Y) % built-in  function
          optimalCritV = inf;
          for j=1: p
              NC= this.InspectedK(j);
              IDX  = this.evalFun(NC);
              if isa(Y,'function_handle')
                  IDX = checkFuncHandle(this,IDX,NC);          
              end
              this.CriterionValues(j)= getDB(this,IDX);
              if  this.CriterionValues(j) < optimalCritV
                  optimalCritV = this.CriterionValues(j);
                  this.OptimalK = this.InspectedK(j);
                  this.OptimalY = IDX;
              end
          end
          
      end
      
      if any(this.Missing) && ~isempty(this.OptimalY)
          this.OptimalY = internal.stats.insertnan(this.Missing,this.OptimalY);
      end
      
      this.PrivX = [];
  
  end
  end %end public methods
  
  methods (Access = protected)
            
      function DB = getDB(this,index)
  
          clusts = unique(index);
          num = length(clusts);
          if num == 1
              DB = nan;
              return;
          end
          centroids = NaN(num,size(this.PrivX,2));
          
          aveWithinD= zeros(num,1);
          for i = 1:num
              members = (index == clusts(i));
              if any(members)
                  centroids(i,:) = mean(this.PrivX(members,:),1) ;
                  %average Euclidean distance of each observation to the centroids
                  aveWithinD(i)= mean(pdist2(this.PrivX(members,:),centroids(i,:)));
              end
          end
                 
          interD = pdist(centroids,'euclidean'); %Euclidean distance
          R = zeros(num);
          for i =1:num
              for j=i+1:num %j>i
                  R(i,j)= (aveWithinD(i)+aveWithinD(j))/ interD((i-1)*(num-i/2)+j-i);%d((I-1)*(M-I/2)+J-I)
              end
          end
          R=R+R';
        
          RI = max(R,[],1);
          DB = mean(RI);
      end
  end %end protected method
  
  end %class
