function p = nctcdf(x,nu,delta,tail)
%MATLAB Code Generation Library Function

%   Copyright 1993-2013 The MathWorks, Inc.
%#codegen

coder.internal.assert(nargin >= 3,'stats:nctcdf:TooFewInputs');
validateattributes(x,{'single','double'},{'real'},'nctpdf','x',1); %#ok<EMCA>
validateattributes(nu,{'numeric'},{'real'},'nctpdf','nu',2); %#ok<EMCA>
validateattributes(delta,{'single','double'},{'real'},'nctpdf','delta',3); %#ok<EMCA>
if nargin < 4
    upper = false;
    tail = 'lower';
else
    coder.internal.prefer_const(tail);
    upper = CheckForUpper(tail);
end
p = coder.internal.sxalloc([true,isfloat(nu),true],x,nu,delta);
for k = 1:numel(p)
    xk = eml_scalexp_subsref(x,k);
    nuk = cast(eml_scalexp_subsref(nu,k),class(p));
    deltak = eml_scalexp_subsref(delta,k);
    if nuk > 0 && ~isnan(deltak) && ~isnan(xk)
        if deltak == 0
            p(k) = tcdf(xk,nuk,tail);
        elseif xk < 0
            p(k) = local_nctcdf(-xk,nuk,-deltak,class(p),~upper);
        else
            p(k) = local_nctcdf(xk,nuk,deltak,class(p),upper);
        end
    else
        p(k) = coder.internal.nan(class(p));
    end
end

%--------------------------------------------------------------------------

function p = local_nctcdf(x,nu,delta,cls,upper)
% Scalar nctcdf for x >= 0, nu > 0, and ~isnan(delta).
ZERO = zeros(cls);
ONE = ones(cls);
if isinf(x)
    if upper
        p = ZERO;
    else
        p = ONE;
    end
    return
end
p = ZERO;
seps = eps(cls);
%Value passed to Incomplete Beta function.
xsq = x*x;
denom = nu + xsq;
P = xsq/denom + ZERO;
Q = nu/denom + ZERO;   % Q = 1-P but avoid roundoff if P is close to 1
% Set up for infinite sum.
dsq = delta*delta;
% Compute probability P[t<0] + P[0<t<x], starting with 1st term
if upper
    if x == 0
        p(1) = normcdf(-delta,ZERO,ONE,'upper');
    end
else
    p(1) = normcdf(-delta,ZERO,ONE);
end
if x > 0
    % Now sum a series to compute the second term
    signd = sign(delta);
    subtotal = ZERO;
    % Start looping over term jj and higher, this should be near the
    % peak of the E part of the term (see below)
    jj = 2*floor(dsq/2);
    % Compute an infinite sum using Johnson & Kotz eq 9, or new
    % edition eq 31.16, each term having this form:
    %      B  = betainc(P,(j+1)/2,nu/2);
    %      E  = (exp(0.5*j*log(0.5*delta^2) - gammaln(j/2+1)));
    %      term = E .* B;
    %
    % We'll compute betainc at the beginning, and then update using
    % recurrence formulas (Abramowitz & Stegun 26.5.16).  We'll sum the
    % series two terms at a time to make the recurrence work out.
    E1 =       exp(0.5*(jj    )*log(0.5*dsq) - dsq/2 - gammaln( jj     /2 + 1));
    E2 = signd*exp(0.5*(jj + 1)*log(0.5*dsq) - dsq/2 - gammaln((jj + 1)/2 + 1));
    % Use either P or Q, whichever is more accurately computed
    if upper
        if P < 0.5
            B1 = real(betainc(P,(jj + 1)/2,nu/2,'upper'));
            B2 = real(betainc(P,(jj + 2)/2,nu/2,'upper'));
        else
            B1 = real(betainc(Q,nu/2,(jj + 1)/2,'lower'));
            B2 = real(betainc(Q,nu/2,(jj + 2)/2,'lower'));
        end
    else
        if P < 0.5
            B1 = real(betainc(P,(jj + 1)/2,nu/2,'lower'));
            B2 = real(betainc(P,(jj + 2)/2,nu/2,'lower'));
        else
            B1 = real(betainc(Q,nu/2,(jj + 1)/2,'upper'));
            B2 = real(betainc(Q,nu/2,(jj + 2)/2,'upper'));
        end
    end
    R1 = exp(gammaln((jj + 1)/2 + nu/2) - gammaln((jj + 3)/2) - gammaln(nu/2) + ...
        ((jj + 1)/2)*log(P) + (nu/2)*log(Q));
    R2 = exp(gammaln((jj + 2)/2+nu/2) - gammaln((jj + 4)/2) - gammaln(nu/2) + ...
        ((jj + 2)/2)*log(P) + (nu/2)*log(Q));
    E10 = E1; E20 = E2; B10 = B1; B20 = B2; R10 = R1; R20 = R2; j0 = jj;
    while true
        %Probability that t lies between 0 and x (x>0)
        twoterms = E1*B1 + E2*B2;
        subtotal = subtotal + twoterms;
        % Convergence test.
        if ~(abs(twoterms) > (abs(subtotal) + seps)*seps)
            break
        end
        % Update for next iteration
        jj = jj + 2;
        E1 = E1* dsq/(jj);
        E2 = E2* dsq/(jj + 1);
        if upper
            B1 = real(betainc(P,(jj + 1)/2,nu/2,'upper'));
            B2 = real(betainc(P,(jj + 2)/2,nu/2,'upper'));
        else
            B1 = B1 - R1;
            B2 = B2 - R2;
            R1 = R1*P*(jj + nu - 1)/(jj + 1);
            R2 = R2*P*(jj + nu    )/(jj + 2);
        end
    end
    % Go back to the peak and start looping downward as far as necessary.
    E1 = E10; E2 = E20; B1 = B10; B2 = B20; R1 = R10; R2 = R20;
    jj = j0;
    while jj > 0
        JJ = jj;
        E1 = E1*(JJ    )/dsq;
        E2 = E2*(JJ + 1)/dsq;
        R1 = R1*(JJ + 1)/((JJ + nu - 1)*P);
        R2 = R2*(JJ + 2)/((JJ + nu    )*P);
        if upper
            B1 = real(betainc(P,(jj - 1)/2,nu/2,'upper'));
            B2 = real(betainc(P,jj/2,nu/2,'upper'));
        else
            B1 = B1 + R1;
            B2 = B2 + R2;
        end
        twoterms = E1*B1 + E2*B2;
        subtotal = subtotal + twoterms;
        if ~(abs(twoterms) > (abs(subtotal) + seps)*seps)
            break
        end
        jj = jj - 2;
    end
    p = min(1,max(0,p + subtotal/2));
end

%--------------------------------------------------------------------------
